/* 
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;

public class ClientConfigDialog extends JDialog {
   private Frame mainWindow;
   private ClientConfigData config;
   private JLabel lblIP, lblPort;
   private JTextField txtIP, txtPort;
   private JButton btnSave, btnClose;
   private JPanel dataPanel, btnPanel;
   
   public ClientConfigDialog(Frame owner){
      super(owner, "Client Config", true);
      
      mainWindow = owner;
      
      config = new ClientConfigData();
      
      readConfigData();
      
      dataPanel = new JPanel(new GridLayout(4, 1));
      lblIP = new JLabel("Server IP:");
      lblPort = new JLabel("Server Port:");
      txtIP = new JTextField();
      txtIP.setText(config.getServerIP());
      txtPort = new JTextField();
      txtPort.setText(String.valueOf(config.getServerPort()));
      dataPanel.add(lblIP);
      dataPanel.add(txtIP);
      dataPanel.add(lblPort);
      dataPanel.add(txtPort);
      
      BtnHandler handler = new BtnHandler();
      
      btnPanel = new JPanel(new GridLayout(1, 2));
      btnSave = new JButton("Save");
      btnSave.setIcon(new ImageIcon(getClass().getResource("res/img/confirm.png")));
      btnSave.setMnemonic('S');
      btnSave.addActionListener(handler);
      btnClose = new JButton("Close");
      btnClose.setIcon(new ImageIcon(getClass().getResource("res/img/closewindow.png")));
      btnClose.setMnemonic('C');
      btnClose.addActionListener(handler);
      btnPanel.add(btnSave);
      btnPanel.add(btnClose);
      
      getContentPane().add(dataPanel, BorderLayout.NORTH);
      getContentPane().add(btnPanel, BorderLayout.SOUTH);
      
      getRootPane().setDefaultButton(btnSave);
      
      setSize(180, 140);
      
      setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
      setLocationRelativeTo(owner);
      setResizable(false);
   }
   
   private void readConfigData() {
      FileInputStream fis = null;
      ObjectInputStream ois = null;
      ClientConfigData tmp;
      
      try {
         fis = new FileInputStream("config.dat");
         ois = new ObjectInputStream(fis);
         tmp = (ClientConfigData) ois.readObject();
         
         config.setServerIP(tmp.getServerIP());
         config.setServerPort(tmp.getServerPort());
         
         ois.close();
         fis.close();
      } catch (Exception e) {
         JOptionPane.showMessageDialog(mainWindow, "Could not read config file. Getting default values!");
      }
      
   }
   
   private void writeConfigData() {
      FileOutputStream fos = null;
      ObjectOutputStream oos = null;
      
      config.setServerIP(txtIP.getText());
      config.setServerPort(Integer.parseInt(txtPort.getText()));
      
      try {
         fos = new FileOutputStream("config.dat");
         oos = new ObjectOutputStream(fos);
         oos.writeObject(config);
         oos.flush();
         oos.close();
         fos.close();
      } catch (Exception e) {
         JOptionPane.showMessageDialog(mainWindow, "Could not write to config file.");
      }
   }
   
   private class BtnHandler implements ActionListener {
      public void actionPerformed(ActionEvent e) {
         if (e.getSource() == btnSave) {
            ClientConfigDialog.this.writeConfigData();
         }
         
         ClientConfigDialog.this.setVisible(false);
         ClientConfigDialog.this.dispose();
      }
   }
}

