/*
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas
 
   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in
   the Software without restriction, including without limitation the rights to
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is furnished to do
   so, subject to the following conditions:
 
   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.
 
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class ContactList {
   private static ImageIcon imgOnline, imgOffline, imgRecvMessage, imgReadMessage;
   private JList contactList;
   private Vector contacts = null;
   private JPopupMenu menu;
   private ServerConnection sc;
   private SoundPlayer sndPlayer;
   
   public ContactList(ServerConnection sc) {
      imgOnline  = new ImageIcon(getClass().getResource("res/img/online.png"));
      imgOffline = new ImageIcon(getClass().getResource("res/img/offline.png"));
      imgRecvMessage = new ImageIcon(getClass().getResource("res/img/letter.png"));
      imgReadMessage = new ImageIcon(getClass().getResource("res/img/openletter.png"));
      this.sc = sc;
      sndPlayer = new SoundPlayer();
      
      contacts = new Vector();
      
      contactList = new JList(contacts);
      contactList.addMouseListener(new ContactListMouseListener());
      contactList.setCellRenderer(new ContactListCellRenderer());
      
      JMenuItem menuItem;
      
      menu = new JPopupMenu();
      
      menuItem = menu.add("Send message");
      menuItem.addActionListener(
      new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            ContactListItem contact;
            
            contact = (ContactListItem)contactList.getSelectedValue();
            if (contact != null)
               contact.showMessageWindow();
         }
      }
      );
      
      menuItem = menu.add("Remove contact");
      menuItem.addActionListener(
      new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            CommData data = new CommData();
            ContactListItem contact = (ContactListItem)contactList.getSelectedValue();
            
            data.setCmd(CommData.CMD_REMOVE_CONTACT_REQ);
            data.setFrom(ContactList.this.sc.getUserID()); // Contact list owner
            data.setTo(contact.getID()); // Contact to be removed
            
            ContactList.this.sc.sendData(data);
         }
      }
      );
   }
   
   public void addContact(ContactListItem c) {
      if (getContactByID(c.getID()) != null)
         return;
      
      contacts.addElement(c);
      //contactList = new JList(contacts);
      contactList.setListData(contacts);
      //sortContactList();
   }
   
   public void removeContact(int cid) {
      ContactListItem tmp = getContactByID(cid);
      contacts.remove(tmp);
      contactList.setListData(contacts);
   }
   
   public JList getList() {
      return contactList;
   }
   
   public void setContactStatus(int cid, int st, boolean playSound) {
      ContactListItem tmp = getContactByID(cid);
      
      if(tmp == null)
         return;
      
      tmp.setStatus(st);
      
      if (playSound) {
         switch (st) {
            case ContactListItem.STATUS_ONLINE:
               sndPlayer.playOnline();
               break;
               
            case ContactListItem.STATUS_OFFLINE:
               sndPlayer.playOffline();
               break;
         }
      }
      
      sortContactList();
   }
   
   private ContactListItem getContactByID(int cid) {
      ContactListItem retVal = null;
      ContactListItem tmp;
      Iterator it = contacts.iterator();
      
      while(it.hasNext()) {
         tmp = (ContactListItem) it.next();
         if(tmp.getID() == cid) {
            retVal = tmp;
            break;
         }
      }
      
      return retVal;
   }
   
   public boolean addMessage(int id, String message) {
      ContactListItem cli = getContactByID(id);
      boolean added = false;
      
      if (cli != null) {
         sndPlayer.playMessage();
         cli.addMessage(message);
         cli.setStatus(ContactListItem.STATUS_MESSAGE_RECEIVED);
         sortContactList();
         added = true;
      }
      
      return added;
   }
   
   public void clearList() {
      contacts.clear();
      contactList.setListData(contacts);
   }
   
   private ImageIcon getContactIcon(ContactListItem c) {
      ImageIcon retVal = null;
      
      switch (c.getStatus()) {
         case ContactListItem.STATUS_MESSAGE_RECEIVED: retVal = imgRecvMessage; break;
         case ContactListItem.STATUS_MESSAGE_READ: retVal = imgReadMessage; break;
         case ContactListItem.STATUS_ONLINE: retVal = imgOnline; break;
         case ContactListItem.STATUS_OFFLINE: retVal = imgOffline; break;
      }
      
      return retVal;
   }
   
   private void sortContactList() {
      Comparator comp = new Comparator() {
         public int compare(Object a, Object b) {
            ContactListItem item1 = (ContactListItem)a;
            ContactListItem item2 = (ContactListItem)b;
            
            return (item1.getStringForOrdering().compareToIgnoreCase(item2.getStringForOrdering()));
         }
      };
      
      Collections.sort(contacts, comp);
      contactList.repaint();
   }
   
   private class ContactListCellRenderer extends DefaultListCellRenderer {
      
      public Component getListCellRendererComponent(JList list,
      Object value,
      int index,
      boolean isSelected,
      boolean hasFocus) {
         JLabel label =
         (JLabel) super.getListCellRendererComponent(list,
         value,
         index,
         isSelected,
         hasFocus);
         
         label.setIcon(getContactIcon((ContactListItem)contacts.get(index)));
         
         return(label);
      }
   }
   
   private class ContactListMouseListener extends MouseAdapter {
      public void mouseClicked(MouseEvent e) {
         ContactListItem contact;
         
         if (e.getClickCount() == 2) {
            contact = (ContactListItem)contactList.getSelectedValue();
            if (contact != null)
               contact.showMessageWindow();
         }
      }
      
      public void mousePressed(MouseEvent e) {
         if (e.isPopupTrigger()) showPopup(e.getPoint());
      }
      
      public void mouseReleased(MouseEvent e) {
         if (e.isPopupTrigger()) showPopup(e.getPoint());
      }
      
      private void showPopup(Point p) {
         int itemindex;
         
         itemindex = contactList.locationToIndex(p);
         
         if (itemindex == -1) return;
         
         contactList.setSelectedIndex(itemindex);
         menu.show(contactList, p.x, p.y);
      }
   }
}
