/* 
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

import java.net.*;
import java.io.*;
import java.util.*;

public class ClientReader extends Thread {
   private ConnectionItem connItem;
   private DBInterface dbInterface;
   private ConnectionsList connList;
   
   public ClientReader(ConnectionItem connItem, DBInterface dbInterface, ConnectionsList connList) {
      this.connItem = connItem;
      this.dbInterface = dbInterface;
      this.connList = connList;
   }
   
   public void run() {
      CommData data = null;
      
      while (true) {
         try {
            data = (CommData) connItem.getReader().readObject();
            if (data != null) {
               new DataHandler(connItem, dbInterface, connList, data).start();
            }
         } catch (Exception e) {
            
            // Now we are treating an user disconnetion:
            
            if ((e instanceof SocketException) || (e instanceof EOFException)) {
               
               int user_id;
                
               try {
                  user_id = dbInterface.getUserFromConnection(connItem.getSocket().getInetAddress().getHostAddress().trim(), connItem.getSocket().getPort());
               } catch (Exception f) {
                  user_id = -1;
               }
                
               // Notify every user who has this user in contact list about his new
               // state (Offline)
               String ownersList;
               CommData notification = new CommData();
                
               try {
                  ownersList = dbInterface.getOwnersList(user_id, true);
               } catch (Exception f) {
                  ownersList = "";
               }
                
               StringTokenizer owners = new StringTokenizer(ownersList, ",");
               int ownerid;
                
               while (owners.hasMoreTokens()) {
                  StringTokenizer owner = new StringTokenizer(owners.nextToken(), ":");
                
                  ownerid = Integer.parseInt(owner.nextToken());
                
                  notification.setCmd(CommData.CMD_USER_IS_OFFLINE);
                  notification.setData(String.valueOf(user_id));
                
                  try {
                       connList.getWriter(dbInterface.getUserIP(ownerid), dbInterface.getUserPort(ownerid)).writeObject(notification);
                       connList.getWriter(dbInterface.getUserIP(ownerid), dbInterface.getUserPort(ownerid)).flush();
                  } catch (Exception f) {
                     // Nothing...
                  }
               }
               
               try {
                  // Update the database for user going offline.
                  dbInterface.setUserOffLine(user_id);
               }
               catch (Exception e2) {
                  // Nothing to do if the user is not online yet...
               }
               
               connList.deleteConnection(connItem);
               // TODO - Check if is it necessary to close the socket here...
               
               break; // Exits thread
               
            } else {
               throw new RuntimeException(e);
            }
         }
      }
   }
}
